from django import forms
import re
from datetime import date

class InfoForm(forms.Form):
    # Validasi Model User
    nama = forms.CharField(
        max_length=255,
        error_messages={
            'required': 'Nama wajib diisi.',  
        }
    )
    def clean_nama(self):
        nama = self.cleaned_data['nama']
        if not re.match(r'^[a-zA-Z0-9._-]+$', nama):
            raise forms.ValidationError("Nama hanya boleh berisi huruf, angka, titik (.), underscore (_), dan tanda hubung (-).")
        return nama

    password = forms.CharField(
        max_length=255,
        error_messages={
            'required': 'Password wajib diisi.',  
        }
    )
    def clean_password(self):
        password = self.cleaned_data['password']
        if len(password) < 8:
            raise forms.ValidationError("Password harus minimal 8 karakter.")
        if not re.match(r'^(?=.*[A-Za-z])(?=.*\d)(?=.*[@$!%*#?&])[A-Za-z\d@$!%*#?&]{8,}$', password):
            raise forms.ValidationError("Password harus mengandung huruf, angka, dan karakter spesial (@$!%*#?&).")
        return password
    
    tanggal_lahir = forms.DateField(
        input_formats=['%Y-%m-%d'],
        error_messages={
            'required': 'Tanggal lahir wajib diisi.',
            'invalid': 'Format tanggal lahir tidak valid. Gunakan format YYYY-MM-DD.',
        }
    )
    def clean_tanggal_lahir(self):
        tanggal_lahir = self.cleaned_data.get('tanggal_lahir')
        today = date.today()
        usia = today.year - tanggal_lahir.year - ((today.month, today.day) < (tanggal_lahir.month, tanggal_lahir.day))
        if usia < 12:
            raise forms.ValidationError("Usia pengguna harus minimal 12 tahun.")
        
        return tanggal_lahir
    
    kode_negara = forms.CharField(
        max_length=3, 
        label="Kode Negara",
        error_messages={
            'required': 'Kode Negara wajib diisi.',  
            'max_length': 'Kode Negara tidak boleh lebih dari 3 digit angka.',
        }
    )
    nomor_hp = forms.CharField(
        max_length=12,
        error_messages={
            'required': 'Nomor HP wajib diisi.', 
            'max_length': 'Nomor HP tidak boleh lebih dari 12 digit angka.',
        }
    )
    def clean_kode_negara(self):
        kode_negara = self.cleaned_data['kode_negara']
        if not re.match(r'^\d{1,3}$', kode_negara):
            raise forms.ValidationError("Kode negara harus berupa angka dengan panjang 1-3 digit.")
        return kode_negara
    def clean_nomor_hp(self):
        nomor_hp = self.cleaned_data['nomor_hp']        
        if not re.match(r'^\d{7,12}$', nomor_hp):
            raise forms.ValidationError("Nomor HP harus berupa angka dengan panjang 7-12 digit.")
        panjang_total = len(nomor_hp)
        if panjang_total < 7:
            raise forms.ValidationError("Panjang nomor HP harus antara 7-12 digit angka.")
        return nomor_hp
    
    email = forms.EmailField(
        error_messages={
            'required': 'Email wajib diisi.',  
        }
    )

    url_blog = forms.URLField(
        error_messages={
            'required': 'URL wajib diisi.',  
        }
    )

    deskripsi_diri = forms.CharField(
        widget=forms.Textarea,
        error_messages={
            'required': 'Deskripsi diri wajib diisi.',  
        }
    )
    def clean_deskripsi_diri(self):
        deskripsi_diri = self.cleaned_data['deskripsi_diri']
        panjang_total = len(deskripsi_diri)
        if panjang_total < 5 or panjang_total > 100:
            raise forms.ValidationError("Deskripsi diri harus minimal 5 karakter dan maksimal 100 karakter.")
        return deskripsi_diri
    
    # Validasi Model Khusus dari Makanan & Minuman
    nik = forms.CharField(
        max_length=16,
        error_messages={
            'required': 'NIK wajib diisi.',  
            'max_length': 'NIK tidak boleh lebih dari 16 digit angka.', 
        }
    )
    def clean_nik(self):
        nik = self.cleaned_data['nik']
        if not re.match(r'^\d{16}$', nik): 
            raise forms.ValidationError("NIK harus berupa 16 digit angka.")
        return nik
    
    poin = forms.IntegerField(  
        min_value=0,
        max_value=1000000,
        error_messages={
            'required': 'Poin wajib diisi.', 
            'min_value': 'Poin tidak boleh kurang dari 0.',
            'max_value': 'Poin tidak boleh lebih dari 1.000.000.',
        }
    )